#version 410 core

layout (location = 0) in vec3 vertex_pos;
layout (location = 1) in vec3 normal_pos;
layout (location = 2) in vec2 vertex_uv;
layout (location = 3) in vec3 vertex_tangent;
layout (location = 4) in vec3 vertex_bitangent;

uniform mat4 m_matrix;
uniform mat4 vp_matrix;
uniform vec3 camera_pos;

out vec2 uv;
out vec3 frag_pos;
out vec3 frag_pos_ts;
out vec3 camera_pos_ts;
out mat3 tbn_mat;

void main(void) {
	frag_pos = (m_matrix * vec4(vertex_pos, 1.0)).xyz;
	gl_Position = vp_matrix * vec4(frag_pos, 1.0);
    uv = vertex_uv;
	
	vec3 T = normalize(mat3(m_matrix) * vertex_tangent);
    vec3 B = normalize(mat3(m_matrix) * vertex_bitangent);
    vec3 N = normalize(mat3(m_matrix) * normal_pos);
	
	// used to get normals from tangent space into world space
    tbn_mat = mat3(T, B, N);
	// used to get frag pos and view pos from world space into tangent space
	mat3 tbn_mat_inv = transpose(tbn_mat);
	
	frag_pos_ts = tbn_mat_inv * frag_pos;
	camera_pos_ts = tbn_mat_inv * camera_pos;
	
}
