#version 410 core

layout (points) in;
layout (triangle_strip) out;
layout (max_vertices = 4) out;

in float curr_frame_gs[];
in float size_gs[];
in float decay_factor_gs[];
in float hue_gs[];

uniform vec3 camera_pos;
uniform mat4 p_matrix;

out vec2 uv;
out float hue;
out float alpha;

void main() {
	vec4 vert_pos = gl_in[0].gl_Position;
	float sprite_dim = 0.1;
	float uv_y_0 = floor((curr_frame_gs[0] / 10.0)) * sprite_dim;
	float uv_y_1 = uv_y_0 + sprite_dim;
	float uv_x_0 = (mod(curr_frame_gs[0], 10.0)) * sprite_dim;
	float uv_x_1 = uv_x_0 + sprite_dim;
	float size = max(0.0, size_gs[0] * (1 - decay_factor_gs[0])) * 0.3f;
	hue = hue_gs[0];
	alpha = 1.0f - decay_factor_gs[0];

	// bottom left
	vec2 va = vert_pos.xy + vec2(-0.5, -0.5) * size;
	gl_Position = p_matrix * vec4(va, vert_pos.zw);
	uv = vec2(uv_x_0, uv_y_0);
	EmitVertex();

	// top left
	vec2 vb = vert_pos.xy + vec2(-0.5, 0.5) * size;
	gl_Position = p_matrix * vec4(vb, vert_pos.zw);
	uv = vec2(uv_x_0, uv_y_1);
	EmitVertex();

	// bottom right
	vec2 vc = vert_pos.xy + vec2(0.5, -0.5) * size;
	gl_Position = p_matrix * vec4(vc, vert_pos.zw);
	uv = vec2(uv_x_1, uv_y_0);
	EmitVertex();

	// top right
	vec2 vd = vert_pos.xy + vec2(0.5, 0.5) * size;
	gl_Position = p_matrix * vec4(vd, vert_pos.zw);
	uv = vec2(uv_x_1, uv_y_1);
	EmitVertex();

	EndPrimitive();
}
