#version 410 core

in vec2 uv;
in float hue;
in float alpha;

uniform sampler2D tex_sampler;

out vec4 frag_color;

vec3 rgb2hsv(vec3 c)
{
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 c)
{
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main() {
	frag_color = texture2D(tex_sampler, uv).rgba;
	if (frag_color.r == 0 && frag_color.g == 0 && frag_color.b == 0 || frag_color.a == 0) discard;
	
	vec3 tex_hsv = rgb2hsv(texture2D(tex_sampler, uv).rgb);
	tex_hsv.r = hue;
	frag_color = vec4(hsv2rgb(tex_hsv), alpha * frag_color.a);
}