#version 410 core

layout(points) in;
layout(points) out;
layout(max_vertices = 30) out;

in float type_gs[];
in vec3 position_gs[];
in vec3 velocity_gs[];
in float lifetime_ms_gs[];
in float decay_factor_gs[];
in float total_lifetime_ms_gs[];
in float curr_frame_gs[];
in float size_gs[];
in float start_hue_gs[];
in float end_hue_gs[];

// those need to match the varyings names defined in gl_programs.yaml
out float type;
out vec3 position;
out vec3 velocity;
out float lifetime_ms;
out float decay_factor;
out float total_lifetime_ms;
out float curr_frame;
out float size;
out float start_hue;
out float end_hue;

uniform float dt_s;
uniform float curr_t_ms;
uniform float particle_lifetime_ms;
uniform float spawn_offset_ms;
uniform int spawn_particle_count;
uniform int spawn_lightsource_count;
uniform vec3 origin_pos;
uniform sampler1D random_directions_sampler;
uniform float start_hue_uniform;
uniform float end_hue_uniform;
uniform float velo_factor;

#define TYPE_ORIGIN 0.0
#define TYPE_LIGHT_SOURCE 1.0
#define TYPE_PARTICLE 2.0

vec3 get_random_direction(float coord) {
	vec3 direction = texture(random_directions_sampler, coord).xyz;
	direction -= vec3(0.5, 0.5, 0.5);
	return vec3(0,0.5f,0) + direction;
}

float get_random(float coord) {
	float value = texture(random_directions_sampler, coord).x;
	return value;
}

void main() {
	float new_lifetime_ms = lifetime_ms_gs[0] + dt_s * 1000.0;
	total_lifetime_ms = total_lifetime_ms_gs[0] + dt_s * 1000.0;

	/*
	 * assume 60fps and skip frames if the framerate cant keep up
	 * note: the animated texture used for particles has 90 frames...
	 */
	curr_frame = mod(floor(total_lifetime_ms / 16.0), 90);

	if (type_gs[0] == TYPE_ORIGIN) {
		// update the origins position and re-emit it
		type = TYPE_ORIGIN;
		position = origin_pos;
		velocity = velocity_gs[0];
		lifetime_ms = new_lifetime_ms > spawn_offset_ms ? 0.0 : new_lifetime_ms;
		decay_factor = 0.0;
		size = size_gs[0];
		start_hue = 0.0f/360.0f;
		end_hue = 0.0f/360.0f;
		EmitVertex();
		EndPrimitive();

		// emit new particles
		while (new_lifetime_ms > spawn_offset_ms) {
			new_lifetime_ms -= spawn_offset_ms;
			for (int i = 0; i < spawn_particle_count; ++i) {
				type = i <= spawn_lightsource_count ? TYPE_LIGHT_SOURCE : TYPE_PARTICLE;
				position = position_gs[0];
				vec3 dir = get_random_direction((0.1f + curr_t_ms + float(i) / spawn_particle_count));
				velocity = normalize(dir) / velo_factor + (get_random((0.2f + curr_t_ms + float(i) / spawn_particle_count)) - 0.5f) * 0.2;
				lifetime_ms = 0.0;
				decay_factor = 0.0;
				size = get_random((0.2f + curr_t_ms + float(i) / spawn_particle_count)) * 0.2 + 0.4;
				start_hue = start_hue_uniform;
				end_hue = end_hue_uniform;
				EmitVertex();
				EndPrimitive();
			}
		}
	} else {
		vec3 d_pos = dt_s * velocity_gs[0]; // pos = t * vel
		vec3 d_vel = dt_s * vec3(0,-length(velocity_gs[0])* 1.2f,0); // gravity

		// update current particle if alive, discard otherwise
		if (new_lifetime_ms < particle_lifetime_ms) {
			type = type_gs[0];
			position = position_gs[0] + d_pos;
			velocity = velocity_gs[0] + d_vel;
			lifetime_ms = new_lifetime_ms;
			decay_factor = new_lifetime_ms / particle_lifetime_ms;
			size = size_gs[0];
			start_hue = start_hue_gs[0];
			end_hue = end_hue_gs[0];
            
            // reflect the particle if it reaches the floor
            if(position.y < 0) {
                velocity = reflect(velocity, vec3(0,1,0)) * 0.5f;
                position.y = 0;
            } if(position.x < -42.25807) {
                velocity = reflect(velocity, vec3(1,0,0)) * 0.5f;
                position.x = -42.25807;
            }
                
			EmitVertex();
			EndPrimitive();
		}
	}
}
